/*
** $Header: /u/spert/src/fltvec/nv/RCS/camfile.c,v 1.5 1996/06/10 20:28:39 davidj Exp $
**
** Cambridge compressed speech file format conversion functions
*/

#include <fltv-config.h>
#include <assert.h>
#include <stddef.h>
/* Note - using Sun SC3.0 acc, files including <assert.h> must also include
   <stdio.h> */
#include <stdio.h>
#include <math.h>
#include <fltv-defs.h>
#include <fltv-scansrc.h>

/*************************************************/
/*** Cambridge "pre-file" conversion functions ***/
/*************************************************/
/*
** The "pre" format is used by the Cambridge speech group to store
** speech features in a file efficiently.  The format uses bytes to encode
** a floating point feature value.
*/

/* Scalar float->pre8 conversion routine. */

#ifdef DECL_nv_htopre8_f_i8
fltv_int8_t nv_htopre8_f_i8(float val);
#endif

#ifdef DEFN_nv_htopre8_f_i8
fltv_int8_t
nv_htopre8_f_i8(float val)
{
    fltv_int32_t res;

    /* FLTV_SQRT1_2 = sqrt(1/2) */
    res = (fltv_int32_t) floor( (erf(val * FLTV_SQRT1_2)+1.0) * 128.0 );
    if (res>FLTV_UINT8_MAX)
	res = FLTV_UINT8_MAX;
    assert(res>=0);
    return ((fltv_int8_t) res);
}
#endif

/* Vector float->pre8 conversion routine. */

#ifdef DECL_nv_htopre8_vf_vi8
void nv_htopre8_vf_vi8(size_t len, const float* from, fltv_int8_t* to);
#endif

#ifdef DEFN_nv_htopre8_vf_vi8
void
nv_htopre8_vf_vi8(size_t len, const float* from, fltv_int8_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to++ = htopre8_f_i8(*from++);
    }
}
#endif

/* Scalar conversion routine. */

#ifdef DECL_nv_pre8toh_i8_f
float nv_pre8toh_i8_f(fltv_int8_t bits);
#endif

#ifdef DEFN_nv_pre8toh_i8_f
float
nv_pre8toh_i8_f(fltv_int8_t bits)
{
    float ret;			/* Return value */
    float* table;		/* Pointer to the start of the lookup table */
    
    table = pre8table_vf();	/* Point to the table for converting pre8
				   bytes to floating point */
    ret = table[(fltv_uint8_t) bits]; /* Perform the lookup */

    return ret;
}
#endif

/*
** This routine converts a vector of 8 bit "pre" value to a floats.
*/

#ifdef DECL_nv_pre8toh_vi8_vf
void nv_pre8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res);
#endif

#ifdef DEFN_nv_pre8toh_vi8_vf
void
nv_pre8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res)
{
    float* table;		/* Pointer to the start of the lookup table */
    size_t i;			/* Local counter */
    
    table = pre8table_vf();	/* Point to the table for converting pre8
				   bytes to floating point */
    for (i=0; i<len; i++)
    {
        *res++ = table[(fltv_uint8_t) *bits++];
    }
}
#endif

/*
** A routine that builds the pre8 to float conversion table
*/

#ifdef DECL_nv_pre8table_vf
float* nv_pre8table_vf();
#endif

#ifdef DEFN_nv_pre8table_vf
float*
nv_pre8table_vf()
{
    enum { TABLEN = 256 };
    static float table[TABLEN];
    static init = 0;

    /* If the tables not build, then build it */
    if (!init)
    {
	size_t i;

        init = 1;
	/* Fill in the values in the table */
	for (i = 0; i<TABLEN; i++)
            table[i] = FLTV_SQRT2 * ierf_f_f(2.0 * (i + 0.5) / TABLEN - 1.0);
    }
    return &table[0];
}
#endif

/*************************************************/
/*** Cambridge "LNA file" conversion functions ***/
/*************************************************/
/*
** The "LNA" format is used by the Cambridge and ICSI speech groups to store
** probabilities efficiently.
** To convert a probability to an 8 bit LNA value, the following formula
** is used:
**
**       lna8 = trunc(-24 * log(x + VERY_SMALL))
*/

#define LNA8_VERY_SMALL (1e-37)
#define LNA8_SCALE (24.0)
 
/* Scalar float->LNA8 conversion routine. */

#ifdef DECL_nv_htolna8_f_i8
fltv_int8_t nv_htolna8_f_i8(float val);
#endif

#ifdef DEFN_nv_htolna8_f_i8
fltv_int8_t
nv_htolna8_f_i8(float val)
{
    fltv_int32_t res;

    res = (int) floor( - LNA8_SCALE * log( val + LNA8_VERY_SMALL ) ) ;
    if (res>FLTV_UINT8_MAX)
	res = FLTV_UINT8_MAX;
    if (res<0)
	res = 0;
    return ((fltv_int8_t) res);
}
#endif

/* Vector float->LNA8 conversion routine. */

#ifdef DECL_nv_htolna8_vf_vi8
void nv_htolna8_vf_vi8(size_t len, const float* from, fltv_int8_t* to);
#endif

#ifdef DEFN_nv_htolna8_vf_vi8
void
nv_htolna8_vf_vi8(size_t len, const float* from, fltv_int8_t* to)
{
    size_t i;

    for (i=0; i<len; i++)
    {
	*to++ = htolna8_f_i8(*from++);
    }
}
#endif


/* Scalar LNA8->float conversion routine. */

#ifdef DECL_nv_lna8toh_i8_f
float nv_lna8toh_i8_f(fltv_int8_t lna);
#endif

#ifdef DEFN_nv_lna8toh_i8_f
float
nv_lna8toh_i8_f(fltv_int8_t lna)
{
    float ret;			/* Return value */
    float* table;		/* Pointer to the start of the lookup table */
    
    table = lna8table_vf();	/* Point to the table for converting lna8
				   bytes to floating point */
    ret = table[(fltv_uint8_t) lna]; /* Perform the lookup */

    return ret;
}
#endif

/*
** This routine converts a vector of 8 bit lna8 value to a floats.
*/

#ifdef DECL_nv_lna8toh_vi8_vf
void nv_lna8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res);
#endif

#ifdef DEFN_nv_lna8toh_vi8_vf
void
nv_lna8toh_vi8_vf(size_t len, const fltv_int8_t* bits, float* res)
{
    float* table;		/* Pointer to the start of the lookup table */
    size_t i;			/* Local counter */
    
    table = lna8table_vf();	/* Point to the table for converting lna8
				   bytes to floating point */
    for (i=0; i<len; i++)
    {
        *res++ = table[(fltv_uint8_t) *bits++];
    }
}
#endif

/*
** A routine that builds the pre8 to float conversion table
*/

#ifdef DECL_nv_lna8table_vf
float* nv_lna8table_vf();
#endif

#ifdef DEFN_nv_lna8table_vf
float*
nv_lna8table_vf()
{
    enum { TABLEN = 256 };
    static float table[TABLEN];
    static init = 0;

    /* If the table is not built, then build it */
    if (!init)
    {
	size_t i;

        init = 1;
	/* Fill in the values in the table */
	for (i = 0; i<TABLEN; i++)
            table[i] = exp(-((double) i + 0.5) / LNA8_SCALE);
    }
    return &table[0];
}
#endif

