/*
   $Header: /u/spert/src/fltvec/testsuite/RCS/mulacc_mfmf_test.c,v 1.2 1995/07/05 23:47:09 davidj Exp $

   Test mulacc_mfmf routines.
*/

#include <stddef.h>
#include <stdlib.h>

#include "fltvec_test.h"

typedef void (*rtn_t)(size_t s1, size_t s2, size_t s3,
		      const float* m1, const float* m2,
		      float* res);

/* Performs one test given input values and expected output. 
   This works for various transpositions - hence the need to supply
   so many arguments */

void
onetest(rtn_t rtn, size_t a_rows, size_t a_cols,
	size_t b_rows, size_t b_cols,
	size_t res_rows, size_t res_cols,
	size_t arg1, size_t arg2, size_t arg3,
	const float* a, const float* b, float* res,
	const float* correct)
{
    /* Create padded copies of inputs. */
    const size_t a_size = a_rows * a_cols;
    const size_t b_size = b_rows * b_cols;
    const size_t res_size = res_rows * res_cols;
    float* c_a = rtst_padvec_clone_vf(a_size, a);
    float* c_b = rtst_padvec_clone_vf(b_size, b);
    float* c_res = rtst_padvec_clone_vf(res_size, res);

    /* Run routine. */
    rtn(arg1, arg2, arg3, c_a, c_b, c_res);

    /* Check result. */
    rtst_checknear_fvfvf(res_size, 1e-5, c_res, correct);

    /* Check input not corrupted. */
    rtst_checkeq_vfvf(a_size, a, c_a);
    rtst_checkeq_vfvf(b_size, b, c_b);

    /* Free and check padding. */
    rtst_padvec_del_vf(c_a);
    rtst_padvec_del_vf(c_b);
    rtst_padvec_del_vf(c_res);
}

void
test_1x1(rtn_t rtn)
{
    static const float a[] = { 2.0f };
    static const float b[] = { 3.0f };
    static float res[] = { 4.0f };
    static const float correct[] = { 10.0f };

    onetest(rtn, 1, 1, 1, 1, 1, 1, 1, 1, 1, a, b, res, correct);
}

void
test_nn_2x3x4(rtn_t rtn)
{
    static const float a[] =
    {
	2.0f, 3.0f, 4.0f,
	5.0f, 6.0f, 7.0f
    };
    static const float b[] =
    {
	0.1f, 0.2f, 0.3f, 0.4f,
	0.5f, 0.6f, 0.7f, 0.8f,
	0.9f, 1.0f, 1.1f, 1.2f
    };
    static float res[] = {
	10.0f, 20.0f, 30.0f, 40.0f,
	50.0f, 60.0f, 70.0f, 80.0f
    };
    static const float correct[] =
    {
	10.0f+2.0f*0.1f+3.0f*0.5f+4.0f*0.9f,
	20.0f+2.0f*0.2f+3.0f*0.6f+4.0f*1.0f,
	30.0f+2.0f*0.3f+3.0f*0.7f+4.0f*1.1f,
	40.0f+2.0f*0.4f+3.0f*0.8f+4.0f*1.2f,
	50.0f+5.0f*0.1f+6.0f*0.5f+7.0f*0.9f,
	60.0f+5.0f*0.2f+6.0f*0.6f+7.0f*1.0f,
	70.0f+5.0f*0.3f+6.0f*0.7f+7.0f*1.1f,
	80.0f+5.0f*0.4f+6.0f*0.8f+7.0f*1.2f,
    };

    onetest(rtn, 2, 3, 3, 4, 2, 4, 2, 3, 4, a, b, res, correct);
}

void
test_nt_2x3x4(rtn_t rtn)
{
    static const float a[] =
    {
	2.0f, 3.0f, 4.0f,
	5.0f, 6.0f, 7.0f
    };
    static const float b[] =
    {
	0.1f, 0.5f, 0.9f,
	0.2f, 0.6f, 1.0f,
	0.3f, 0.7f, 1.1f,
	0.4f, 0.8f, 1.2f
    };
    static float res[] = {
	10.0f, 20.0f, 30.0f, 40.0f,
	50.0f, 60.0f, 70.0f, 80.0f
    };
    static const float correct[] =
    {
	10.0f+2.0f*0.1f+3.0f*0.5f+4.0f*0.9f,
	20.0f+2.0f*0.2f+3.0f*0.6f+4.0f*1.0f,
	30.0f+2.0f*0.3f+3.0f*0.7f+4.0f*1.1f,
	40.0f+2.0f*0.4f+3.0f*0.8f+4.0f*1.2f,
	50.0f+5.0f*0.1f+6.0f*0.5f+7.0f*0.9f,
	60.0f+5.0f*0.2f+6.0f*0.6f+7.0f*1.0f,
	70.0f+5.0f*0.3f+6.0f*0.7f+7.0f*1.1f,
	80.0f+5.0f*0.4f+6.0f*0.8f+7.0f*1.2f,
    };

    onetest(rtn, 2, 3, 4, 3, 2, 4, 2, 3, 4, a, b, res, correct);
}

void
test_tn_2x3x4(rtn_t rtn)
{
    static const float a[] =
    {
	2.0f, 5.0f,
	3.0f, 6.0f,
	4.0f, 7.0f
    };
    static const float b[] =
    {
	0.1f, 0.2f, 0.3f, 0.4f,
	0.5f, 0.6f, 0.7f, 0.8f,
	0.9f, 1.0f, 1.1f, 1.2f
    };
    static float res[] = {
	10.0f, 20.0f, 30.0f, 40.0f,
	50.0f, 60.0f, 70.0f, 80.0f
    };
    static const float correct[] =
    {
	10.0f+2.0f*0.1f+3.0f*0.5f+4.0f*0.9f,
	20.0f+2.0f*0.2f+3.0f*0.6f+4.0f*1.0f,
	30.0f+2.0f*0.3f+3.0f*0.7f+4.0f*1.1f,
	40.0f+2.0f*0.4f+3.0f*0.8f+4.0f*1.2f,
	50.0f+5.0f*0.1f+6.0f*0.5f+7.0f*0.9f,
	60.0f+5.0f*0.2f+6.0f*0.6f+7.0f*1.0f,
	70.0f+5.0f*0.3f+6.0f*0.7f+7.0f*1.1f,
	80.0f+5.0f*0.4f+6.0f*0.8f+7.0f*1.2f,
    };

    onetest(rtn, 3, 2, 3, 4, 2, 4, 3, 2, 4, a, b, res, correct);
}

void
test_tt_2x3x4(rtn_t rtn)
{
    static const float a[] =
    {
	2.0f, 5.0f,
	3.0f, 6.0f,
	4.0f, 7.0f
    };
    static const float b[] =
    {
	0.1f, 0.5f, 0.9f,
	0.2f, 0.6f, 1.0f,
	0.3f, 0.7f, 1.1f,
	0.4f, 0.8f, 1.2f
    };
    static float res[] = {
	10.0f, 20.0f, 30.0f, 40.0f,
	50.0f, 60.0f, 70.0f, 80.0f
    };
    static const float correct[] =
    {
	10.0f+2.0f*0.1f+3.0f*0.5f+4.0f*0.9f,
	20.0f+2.0f*0.2f+3.0f*0.6f+4.0f*1.0f,
	30.0f+2.0f*0.3f+3.0f*0.7f+4.0f*1.1f,
	40.0f+2.0f*0.4f+3.0f*0.8f+4.0f*1.2f,
	50.0f+5.0f*0.1f+6.0f*0.5f+7.0f*0.9f,
	60.0f+5.0f*0.2f+6.0f*0.6f+7.0f*1.0f,
	70.0f+5.0f*0.3f+6.0f*0.7f+7.0f*1.1f,
	80.0f+5.0f*0.4f+6.0f*0.8f+7.0f*1.2f,
    };

    onetest(rtn, 3, 2, 4, 3, 2, 4, 3, 2, 4, a, b, res, correct);
}

void
directed_nn(void)
{
    rtst_log("starting directed tests.\n");
    test_1x1(nv_mulacc_mfmf_mf);
    test_1x1(php_mulacc_mfmf_mf);
    test_nn_2x3x4(nv_mulacc_mfmf_mf);
    test_nn_2x3x4(php_mulacc_mfmf_mf);
    rtst_log("finished directed tests.\n");
}

void
directed_nt(void)
{
    rtst_log("starting directed tests.\n");
    test_1x1(nv_mulntacc_mfmf_mf);
    test_1x1(php_mulntacc_mfmf_mf);
    test_nt_2x3x4(nv_mulntacc_mfmf_mf);
    test_nt_2x3x4(php_mulntacc_mfmf_mf);
    rtst_log("finished directed tests.\n");
}

void
directed_tn(void)
{
    rtst_log("starting directed tests.\n");
    test_1x1(nv_multnacc_mfmf_mf);
    test_tn_2x3x4(nv_multnacc_mfmf_mf);
    test_1x1(php_multnacc_mfmf_mf);
    test_tn_2x3x4(php_multnacc_mfmf_mf);
    rtst_log("finished directed tests.\n");
}

void
directed_tt(void)
{
    rtst_log("starting directed tests.\n");
    test_1x1(nv_multtacc_mfmf_mf);
    test_1x1(php_multtacc_mfmf_mf);
    test_tt_2x3x4(nv_multtacc_mfmf_mf);
    test_tt_2x3x4(php_multtacc_mfmf_mf);
    rtst_log("finished directed tests.\n");
}

int
main(int argc, char *argv[])
{
    rtst_args(argc, argv);

    rtst_start("mulacc_mfmf_mf");
    directed_nn();
    rtst_passed();

    rtst_start("mulntacc_mfmf_mf");
    directed_nt();
    rtst_passed();

    rtst_start("multnacc_mfmf_mf");
    directed_tn();
    rtst_passed();

    rtst_start("multtacc_mfmf_mf");
    directed_tt();
    rtst_passed();

    rtst_exit();

    return 0;
}
